import re

from django.core.exceptions import ObjectDoesNotExist
from django.db import models

class CreateTimeBaseObject(models.Model):
    created = models.DateTimeField(auto_now_add=True, db_index=True)
    modified = models.DateTimeField(auto_now=True, db_index=True)
    active = models.BooleanField(default=True, db_index=True)

    class Meta:
        abstract = True


class Release(CreateTimeBaseObject):
    name = models.CharField(max_length=255)
    order = models.IntegerField(max_length=3, default=0, db_index=True)

    def last_ten_builds(self):
        return self.builds.all().order_by('-created')[:10]

    class Meta:
        ordering = ('-order', '-created')

class Build(CreateTimeBaseObject):
    name = models.CharField(max_length=255)
    release = models.ForeignKey(Release, related_name='builds')
    description = models.TextField()
    status = models.CharField(max_length=16)

    def __getattr__(self, name):
        if name in ['build', 'publish', 'bvt_1', 'bvt_2']:
            try:
                return self.stages.get(
                    name=name
                )
            except ObjectDoesNotExist:
                return None
        return super(Build, self).__getattr__(name)

    def _get_links(self):
        links = re.findall(r'http://.*?\.(?:lrz|iso)\.torrent', self.description)
        return {i.split('.')[-2]:i for i in links}

    @property
    def iso(self):
        links = self._get_links()
        return links.get('iso')

    @property
    def upgrade(self):
        links = self._get_links()
        return links.get('lrz')


class Stage(CreateTimeBaseObject):
    name = models.CharField(max_length=255)
    build = models.ForeignKey(Build, related_name='stages')
    url = models.CharField(max_length=255, default='',
                           null=True, blank=True)
    value = models.CharField(max_length=255)


class Group(models.Model):
    name = models.CharField(max_length=255)
    releases = models.ManyToManyField(Release, related_name='+')
