from django.shortcuts import HttpResponse, render_to_response, get_object_or_404
from django.views.decorators.csrf import csrf_exempt
from release import settings
from sheet.models import Build, Release, Stage, Group
from security.decorators import token_auth


import json
import yaml

order_acceptable_values = [
    'name',
    'created',
    'modified',
    'order',
    '-name',
    '-created',
    '-modified',
    '-order'
]


def index(request, outformat, group=None):
    release_order = request.GET.get('release_order')
    if release_order not in order_acceptable_values:
        release_order = '-name'

    if not group:
        group = settings.DEFAULT_SCREEN

    visible_releases = get_object_or_404(Group, name=group).releases.all().values('id')

    releases = Release.objects.filter(
        active=True,
        id__in=[item['id'] for item in visible_releases]
    ).order_by(release_order)

    if outformat:
        data = {}
        for release in releases:
            if release.name not in data:
                data[str(release.name)] = []

            for build in release.last_ten_builds():
                data[str(release.name)].append({
                    'name': str(build.name),
                    'status': str(build.status),
                    'created': str(build.created),
                    'build_url': str(build.build.url) if build.build else None,
                    'publish_url': str(build.publish.url) if build.publish else None,
                    'bvt_1_url': str(build.bvt_1.url) if build.bvt_1 else None,
                    'bvt_2_url': str(build.bvt_2.url) if build.bvt_2 else None,
                    'description': str(build.description),
                })

        if outformat == '.json':
            return HttpResponse(
                content=json.dumps(data),
                content_type='application/json')
        elif outformat == '.yaml':
            return HttpResponse(
                content=yaml.dump(data, default_flow_style=False),
                content_type='application/yaml')
    else:
        return render_to_response(
            'sheet/index.html', {
                'releases': releases,
            },
        )


@csrf_exempt
@token_auth
def add_stage(request, release, build):
    if request.method == 'POST':
        r, r_c = Release.objects.get_or_create(
            name=release)
        b, b_c = Build.objects.get_or_create(
            name=build, release=r)
        s, s_c = Stage.objects.get_or_create(
            name=request.POST.get('name'),
            build=b
        )

        s.url = request.POST.get('url')
        s.value = request.POST.get('value')
        s.save()

        if request.POST.get('description'):
            b.description = request.POST.get('description')
            b.save()

        stages = Stage.objects.filter(build=b)

        i = 0
        for stage in stages:
            if stage.value == 'fail':
                i+=1

        if i == 3 or i == 2:
            b.status = 'danger'
            b.save()
        elif i == 1:
            b.status = 'warning'
            b.save()
        elif i == 0:
            b.status = 'success'
            b.save()

        return HttpResponse(
            content=json.dumps({
                'status': 'ok',
            }),
            content_type='application/json'
        )
    else:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'message': 'request method should be POST',
            }),
            content_type='application/json',
            status='400',
        )
